from bs4 import BeautifulSoup
from compiler.ast import flatten

from robostrippy.http import Fetcher


class attr(object):
    def __init__(self, selectors, attribute=None, all=False):
        self.selectors = selectors
        if not isinstance(self.selectors, list):
            self.selectors = [self.selectors]
        self.attribute = attribute
        self.all = all

    def text(self, elem):
        return "".join([s.strip() for s in elem.strings])

    def __get__(self, obj, objtype):
        matches = obj._content.select(self.selectors[0])
        for selector in self.selectors[1:]:
            matches = flatten([match.select(selector) for match in matches])
        if len(matches) == 0:
            return None
        if self.attribute is not None:
            return matches[0].get(self.attribute)
        if self.all:
            return [self.text(elem) for elem in matches]
        return self.text(matches[0])


class attrList(object):
    def __init__(self, selector, klass):
        self.selector = selector
        self.klass = klass

    def __get__(self, obj, objtype):
        matches = obj._content.select(self.selector)
        return [self.klass(None, match) for match in matches]


class Resource:
    def __init__(self, url, content=None, headers=None):
        self._url = url
        self._content = content
        if self._content is None:
            html = Fetcher.get(self._url)
            self._content = BeautifulSoup(html)

    def __str__(self):
        props = []
        for key in dir(self):
            if not key.startswith("_"):
                props.append("%s='%s'" % (key, getattr(self, key)))
        return "<%s %s>" % (self.__class__.__name__, ", ".join(props))

    __repr__ = __str__



import requests

USERAGENT_HEADER = """Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) \
AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.101 Safari/537.11"""
ACCEPT_HEADER = """text/html,application/xhtml+xml,application/xml;q=0.9,\
*/*;q=0.8"""


class Fetcher:
    @classmethod
    def get(self, url, headers=None):
        default_headers = {
            'Accept': ACCEPT_HEADER,
            'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
            'Accept-Language': 'en-US,en;q=0.8',
            'User-Agent': USERAGENT_HEADER
            }
        default_headers.update(headers or {})
        req = requests.get(url, headers=default_headers)
        return req.text



#!/usr/bin/env python
from setuptools import setup, find_packages

from robostrippy import version

setup(
    name="robostrippy",
    version=version,
    description="Strip HTML websites as though they provided a REST interface.  Like a Robot.",
    author="Brian Muller",
    author_email="bamuller@gmail.com",
    license="MIT",
    url="http://github.com/bmuller/robostrippy",
    packages=find_packages(),
    requires=["bs4.BeautifulSoup", "requests"],
    install_requires=["beautifulsoup4 >= 4.3.1", "requests >= 1.2.3"]
)
